package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeCacheStrategy;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivationEventType;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.model.AbstractUtility;

/**
 * Context menu for the creation of sub-derivates.
 *
 */
public class CreateDerivateContextMenu extends CompoundContributionItem {

    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
     */
    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        final ISelection selection = window.getActivePage().getSelection();
                        if(selection instanceof TreeSelection){
                            final Object selectedElement = ((TreeSelection) selection).getFirstElement();
                            //context menu for FieldUnit
                            if(selectedElement instanceof FieldUnit){
                                FieldUnit fieldUnit = (FieldUnit) selectedElement;
                                createMenuItem(menu, "Specimen", fieldUnit, DerivedUnit.class, SpecimenOrObservationType.PreservedSpecimen);
                                new MenuItem(menu, SWT.SEPARATOR);
                                createMenuItem(menu, "Tissue Sample", fieldUnit, DerivedUnit.class, SpecimenOrObservationType.TissueSample);
                                createMenuItem(menu, "DNA Sample", fieldUnit, DnaSample.class, SpecimenOrObservationType.DnaSample);
                                createMenuItem(menu, "Specimen Scan", fieldUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                                createMenuItem(menu, "Artwork", fieldUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                                createMenuItem(menu, "Living Plant Photo", fieldUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                            }
                            //context menu for Specimen
                            else if(selectedElement instanceof DerivedUnit && ((DerivedUnit)selectedElement).getRecordBasis()==SpecimenOrObservationType.PreservedSpecimen){
                                DerivedUnit derivedUnit = (DerivedUnit) selectedElement;

                                createMenuItem(menu, "Tissue Sample", derivedUnit, DerivedUnit.class, SpecimenOrObservationType.TissueSample);
                                createMenuItem(menu, "DNA Sample", derivedUnit, DnaSample.class, SpecimenOrObservationType.DnaSample);
                                createMenuItem(menu, "Specimen Scan", derivedUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                                createMenuItem(menu, "Artwork", derivedUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                                createMenuItem(menu, "Living Plant Photo", derivedUnit, MediaSpecimen.class, SpecimenOrObservationType.StillImage);
                            }
                            //context menu for DnaSample
                            else if(selectedElement instanceof DnaSample){
                                DnaSample dnaSample = (DnaSample) selectedElement;
                                createMenuItem(menu, "Consensus Sequence", dnaSample, Sequence.class, null);
                                createMenuItem(menu, "Amplification", dnaSample, Amplification.class, null);
                            }
                            //context menu for Amplification
                            else if(selectedElement instanceof Amplification){
                                Amplification amplification = (Amplification) selectedElement;
                                createMenuItem(menu, "Single Read", amplification, SingleRead.class, null);
                            }
                            //context menu for Sequence
                            else if(selectedElement instanceof Sequence){
                                Sequence sequence = (Sequence) selectedElement;
                                createMenuItem(menu, "Single Read", sequence, SingleRead.class, null);
                            }
                            //default
                            else{
                                MenuItem menuItem = new MenuItem(menu, SWT.None);
                                menuItem.setText("No child derivates");
                                menuItem.setEnabled(false);
                            }
                        }
                    }

                    private void createMenuItem(Menu menu, String menuLabel, Object parent, Class<?> childClass, SpecimenOrObservationType specimenType) {
                        MenuItem menuItem = new MenuItem(menu, SWT.None);
                        menuItem.setText(menuLabel);
                        menuItem.addSelectionListener(new WidgetSelectionListener(parent, childClass, specimenType));
                    }

                }
        };
        return contributionItems;
    }

    private class WidgetSelectionListener implements SelectionListener{

        private final Object parent;
        //TODO: we need a common interface for the generic like e.g. "IDerivateHierarchyNode"
        private final Class<?> childClass;
        private final SpecimenOrObservationType specimenType;

        /**
         * @param parent
         * @param specimenType
         */
        public WidgetSelectionListener(Object parent, Class<?> childClass, SpecimenOrObservationType specimenType) {
            this.parent = parent;
            this.childClass = childClass;
            this.specimenType = specimenType;
        }

        /* (non-Javadoc)
         * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetSelected(SelectionEvent e) {
            if(parent instanceof DnaSample){
                DnaSample dnaSample = (DnaSample)parent;
                if(childClass.equals(Amplification.class)){
                    Amplification.NewInstance(dnaSample);
                }
                else if(childClass.equals(Sequence.class)){
                    dnaSample.addSequence(Sequence.NewInstance(""));//why does this not have DnaSample as constructor parameter?
                }
            }
            else if(parent instanceof Amplification){
                ((Amplification) parent).addSingleRead(SingleRead.NewInstance());
            }
            else if(parent instanceof Sequence){
                ((Sequence) parent).addSingleRead(SingleRead.NewInstance());
            }
            else if(parent instanceof SpecimenOrObservationBase && DerivedUnit.class.isAssignableFrom(childClass)){
                SpecimenOrObservationBase<?> specimenOrObservationBase = (SpecimenOrObservationBase<?>)parent;
                DerivedUnit derivedUnit;
                if(childClass.equals(DnaSample.class)){
                    derivedUnit = DnaSample.NewInstance();
                }
                else if(childClass.equals(MediaSpecimen.class)){
                    derivedUnit = MediaSpecimen.NewInstance(specimenType);
                    //a MediaSpecimen with no media attached does not make sense hence we add one
                    ((MediaSpecimen)derivedUnit).setMediaSpecimen(Media.NewInstance());
                }
                else{
                    derivedUnit = DerivedUnit.NewInstance(specimenType);
                }
                specimenOrObservationBase.addDerivationEvent(DerivationEvent.NewSimpleInstance(specimenOrObservationBase, derivedUnit, DerivationEventType.GATHERING_IN_SITU()));
                derivedUnit.setCacheStrategy(new DerivedUnitFacadeCacheStrategy());
                derivedUnit.getTitleCache(); //update title cache
            }
            IWorkbenchPart activePart = AbstractUtility.getActivePart();
            if(activePart instanceof DerivateView){
                DerivateView derivateView = (DerivateView)activePart;
                derivateView.changed(null);
                derivateView.viewer.refresh();
            }
        }

        /* (non-Javadoc)
         * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetDefaultSelected(SelectionEvent e) {
            // TODO Auto-generated method stub

        }

    }

}
