package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.derivateSearch.DerivateLabelProvider;

/**
 * Displays the derivate hierarchy of all derivates belonging to a taxon.
 *
 */
public class DerivateView extends EditorPart implements IPartContentHasFactualData, IDirtyMarkableSelectionProvider, IConversationEnabled, IPartContentHasDetails, IPartContentHasSupplementalData {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.view.desc"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView";

	private ConversationHolder conversation;

	protected TreeViewer viewer;

    private boolean isDirty;

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createPartControl(Composite parent) {
        viewer = new TreeViewer(new Tree(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION));
        viewer.setContentProvider(new DerivateContentProvider());
        viewer.setLabelProvider(new DerivateLabelProvider());
        viewer.setAutoExpandLevel(AbstractTreeViewer.ALL_LEVELS);
        // Propagate selection from viewer
        getSite().setSelectionProvider(viewer);

        //create context menu
        MenuManager menuManager = new MenuManager();

        getSite().registerContextMenu(menuManager, viewer);

        Control control = viewer.getControl();
        Menu menu = menuManager.createContextMenu(control);

        control.setMenu(menu);

        IEditorInput editorInput = getEditorInput();
        viewer.setInput(editorInput);
        if(editorInput instanceof DerivateViewEditorInput){
            SpecimenOrObservationBase<?> specimen = ((DerivateViewEditorInput) editorInput).getDerivate();
            SpecimenOrObservationBase rootElement = ((DerivateViewEditorInput) editorInput).getRootElement();
            setPartName(rootElement.toString());
            viewer.setSelection(new StructuredSelection(specimen));
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public void doSave(IProgressMonitor monitor) {
        monitor.beginTask("Saving Derivates", 3);
        if (!conversation.isBound()) {
            conversation.bind();
        }
        monitor.worked(1);

        // commit the conversation and start a new transaction immediately
        conversation.commit(true);
        monitor.worked(1);

        this.setDirty(false);
        monitor.worked(1);
        monitor.done();
        firePropertyChange(PROP_DIRTY);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#doSaveAs()
     */
    @Override
    public void doSaveAs() {
        // TODO Auto-generated method stub

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#getTitleToolTip()
     */
    @Override
    public String getTitleToolTip() {
        return "Derivate View";
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
     */
    @Override
    public void init(IEditorSite site, IEditorInput input) throws PartInitException {
        setSite(site);
        setInput(input);
        if(input instanceof DerivateViewEditorInput){
            conversation = ((DerivateViewEditorInput) input).getConversationHolder();
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#isDirty()
     */
    @Override
    public boolean isDirty() {
        return isDirty;
    }

    /**
     * @param isDirty the isDirty to set
     */
    public void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
     */
    @Override
    public boolean isSaveAsAllowed() {
        // TODO Auto-generated method stub
        return false;
    }


    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
     */
    @Override
    public void setFocus() {
        //make sure to bind again if maybe in another view the conversation was unbound
        if(!conversation.isBound()){
            conversation.bind();
        }
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
     */
    @Override
    public void update(CdmDataChangeMap changeEvents) {
        // TODO Auto-generated method stub

    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
     */
    @Override
    public ConversationHolder getConversationHolder() {
        return conversation;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#changed(java.lang.Object)
     */
    @Override
    public void changed(Object element) {
        viewer.setInput(getEditorInput());
        setDirty(true);
        firePropertyChange(IEditorPart.PROP_DIRTY);
    }

}
