// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.derivate;

import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Editor input for the {@link DerivateView} which holds the currently selected derivate for which
 * the derivate hierarchy should be shown in the DerivateView.<br>
 * It also holds a {@link SpecimenOrObservationBase} which is the root of the hierarchy. (both may be the same object)
 * @author pplitzner
 * @date 25.11.2013
 *
 */
public class DerivateViewEditorInput implements IEditorInput {

    /**
     * The selected derivate
     */
    private final SpecimenOrObservationBase<?> derivate;
    /**
     * The root of the hierarchy (may be the same object as the derivate)
     */
    private SpecimenOrObservationBase<?> root;

    private final ConversationHolder conversationHolder;

    /**
     * Creates an editor input for the {@link DerivateView} with the currently selected derivate and the
     * corresponding {@link FieldUnit} (both may be the same object).
     * @param derivateUuid the {@link UUID} of the derivate for which the derivate hierarchy should be shown
     * @param root the root of the hierarchy
     */
    public DerivateViewEditorInput(UUID derivateUuid) {
        super();
        this.conversationHolder = CdmStore.createConversation();
        this.derivate = CdmStore.getCurrentApplicationConfiguration().getOccurrenceService().load(derivateUuid);
        if(derivate instanceof FieldUnit){
            this.root = derivate;
        }
        else if(derivate instanceof DerivedUnit){
            root = EditorUtil.getTopMostDerivate(derivate);
            if(root==null){
                root = derivate;
            }
        }
        if(root==null){
            AbstractUtility.errorDialog("Failed initializing editor", DerivateViewEditorInput.class, "No root element found!");
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    @Override
    public Object getAdapter(Class adapter) {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#exists()
     */
    @Override
    public boolean exists() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
     */
    @Override
    public ImageDescriptor getImageDescriptor() {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getName()
     */
    @Override
    public String getName() {
        return derivate.toString();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getPersistable()
     */
    @Override
    public IPersistableElement getPersistable() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getToolTipText()
     */
    @Override
    public String getToolTipText() {
        return derivate.toString();
    }


    /**
     * @return the specimen
     */
    public SpecimenOrObservationBase<?> getDerivate() {
        return derivate;
    }

    /**
     * @return the fieldUnit
     */
    public SpecimenOrObservationBase<?> getRootElement() {
        return root;
    }

    /**
     * @return the conversationHolder
     */
    public ConversationHolder getConversationHolder() {
        return conversationHolder;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((root == null) ? 0 : root.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        DerivateViewEditorInput other = (DerivateViewEditorInput) obj;
        if (root == null) {
            if (other.root != null) {
                return false;
            }
        } else if (!root.equals(other.root)) {
            return false;
        }
        return true;
    }

}
